<?php

use yii\db\Schema;
use yii\db\Migration;

class m150310_100557_create_product_table extends Migration
{
    public function up()
    {
        $tableExist = $this->db->getTableSchema("{{%catalog_product}}", true);
        if ($tableExist)
        {
            return true;
        }

        $tableOptions = null;
        if ($this->db->driverName === 'mysql') {
            $tableOptions = 'CHARACTER SET utf8 COLLATE utf8_general_ci ENGINE=InnoDB';
        }

        $this->createTable("{{%catalog_product}}", [
            'id'                    => Schema::TYPE_PK,

            'created_by'            => Schema::TYPE_INTEGER . ' NULL',
            'updated_by'            => Schema::TYPE_INTEGER . ' NULL',

            'created_at'            => Schema::TYPE_INTEGER . ' NULL',
            'updated_at'            => Schema::TYPE_INTEGER . ' NULL',
            'published_at'          => Schema::TYPE_INTEGER . ' NULL',

            'name'                  => Schema::TYPE_STRING. '(255) NOT NULL',

            'description_short'     => Schema::TYPE_TEXT,
            'description_full'      => Schema::TYPE_TEXT,

            'files'                 => Schema::TYPE_TEXT. ' NULL', //

            'seo_page_name'         => Schema::TYPE_STRING. '(64) NOT NULL', //обложка

            'count_comment'         => Schema::TYPE_INTEGER . ' NULL', //Количество комментариев

            'count_subscribe'       => Schema::TYPE_INTEGER . ' NULL', //Количество подписчиков
            'users_subscribers'     => Schema::TYPE_TEXT. ' NULL',   //Пользователи которые подписались (их id через запятую)


            'count_vote'            => Schema::TYPE_INTEGER . ' NULL', //Количество голосов
            'result_vote'           => Schema::TYPE_INTEGER . ' NULL', //Результат голосования
            'users_votes_up'        => Schema::TYPE_TEXT. ' NULL',   //Пользователи которые проголосовали +
            'users_votes_down'      => Schema::TYPE_TEXT. ' NULL',   //Пользователи которые проголосовали -

            'status'                => Schema::TYPE_SMALLINT . ' NOT NULL DEFAULT 10', //статус, активна некативна, удалено
            'status_adult'          => Schema::TYPE_SMALLINT . ' NOT NULL DEFAULT 0', //Возрастной статус 0 - не проверено, 1-для всех, 2-типо эротические материалы, 3-порно

            'page_options'          => Schema::TYPE_TEXT. ' NULL',

            'tree_id'               => Schema::TYPE_INTEGER. ' NULL',
            'tree_ids'              => Schema::TYPE_STRING. '(500) NULL',
            'type'                  => Schema::TYPE_STRING. '(32) NULL',

            'availability'          => Schema::TYPE_INTEGER. ' NULL', //наличие

            'ammount'                               => Schema::TYPE_STRING. '(255) NULL',                   //значение цены
            'currency'                              => Schema::TYPE_STRING. '(3) NULL DEFAULT "RUB"',         //валюта цены

            'ammount_procurement'                   => Schema::TYPE_STRING. '(255) NULL',                   //Закупочная цена
            'currency_procurement'                  => Schema::TYPE_STRING. '(3) NULL DEFAULT "RUB"',                     //Закупочная цена

            'ammount_old'                           => Schema::TYPE_STRING. '(255) NULL',                   //Старая цена, например чтобы показывать зачеркнутую цену
            'currency_old'                          => Schema::TYPE_STRING. '(3) NULL DEFAULT "RUB"',

            'ammount_additional'                           => Schema::TYPE_STRING. '(255) NULL',
            'currency_additional'                          => Schema::TYPE_STRING. '(3) NULL DEFAULT "RUB"',

            'article'                           => Schema::TYPE_STRING. '(255) NULL',

        ], $tableOptions);

        $this->execute("ALTER TABLE {{%catalog_product}} ADD INDEX(updated_by);");
        $this->execute("ALTER TABLE {{%catalog_product}} ADD INDEX(created_by);");

        $this->execute("ALTER TABLE {{%catalog_product}} ADD INDEX(created_at);");
        $this->execute("ALTER TABLE {{%catalog_product}} ADD INDEX(updated_at);");
        $this->execute("ALTER TABLE {{%catalog_product}} ADD INDEX(published_at);");

        $this->execute("ALTER TABLE {{%catalog_product}} ADD INDEX(name);");
        $this->execute("ALTER TABLE {{%catalog_product}} ADD UNIQUE(seo_page_name);");

        $this->execute("ALTER TABLE {{%catalog_product}} ADD INDEX(count_comment);");
        $this->execute("ALTER TABLE {{%catalog_product}} ADD INDEX(count_subscribe);");
        $this->execute("ALTER TABLE {{%catalog_product}} ADD INDEX(count_vote);");
        $this->execute("ALTER TABLE {{%catalog_product}} ADD INDEX(result_vote);");

        $this->execute("ALTER TABLE {{%catalog_product}} ADD INDEX(status);");
        $this->execute("ALTER TABLE {{%catalog_product}} ADD INDEX(status_adult);");

        $this->execute("ALTER TABLE {{%catalog_product}} ADD INDEX(availability);");
        $this->execute("ALTER TABLE {{%catalog_product}} ADD INDEX(type);");
        $this->execute("ALTER TABLE {{%catalog_product}} ADD INDEX(tree_id);");


        $this->execute("ALTER TABLE {{%catalog_product}} ADD INDEX(ammount);");
        $this->execute("ALTER TABLE {{%catalog_product}} ADD INDEX(currency);");

        $this->execute("ALTER TABLE {{%catalog_product}} ADD INDEX(ammount_procurement);");
        $this->execute("ALTER TABLE {{%catalog_product}} ADD INDEX(currency_procurement);");

        $this->execute("ALTER TABLE {{%catalog_product}} ADD INDEX(ammount_old);");
        $this->execute("ALTER TABLE {{%catalog_product}} ADD INDEX(currency_old);");

        $this->execute("ALTER TABLE {{%catalog_product}} ADD INDEX(ammount_additional);");
        $this->execute("ALTER TABLE {{%catalog_product}} ADD INDEX(currency_additional);");

        $this->execute("ALTER TABLE {{%catalog_product}} ADD UNIQUE(article);");

        $this->execute("ALTER TABLE {{%catalog_product}} COMMENT = 'Каталог продукт';");


        $this->addForeignKey(
            'catalog_product_created_by', "{{%catalog_product}}",
            'created_by', '{{%cms_user}}', 'id', 'RESTRICT', 'RESTRICT'
        );

        $this->addForeignKey(
            'catalog_product_updated_by', "{{%catalog_product}}",
            'updated_by', '{{%cms_user}}', 'id', 'RESTRICT', 'RESTRICT'
        );

        $this->addForeignKey(
            'catalog_product_tree_id_cms_tree', "{{%catalog_product}}",
            'tree_id', '{{%cms_tree}}', 'id', 'RESTRICT', 'RESTRICT'
        );


        $this->addForeignKey(
            'catalog_product_currency__money_currency', "{{%catalog_product}}",
            'currency', '{{%money_currency}}', 'code', 'RESTRICT', 'RESTRICT'
        );

        $this->addForeignKey(
            'catalog_product_currency_procurement__money_currency', "{{%catalog_product}}",
            'currency_procurement', '{{%money_currency}}', 'code', 'RESTRICT', 'RESTRICT'
        );
        $this->addForeignKey(
            'catalog_product_currency_old__money_currency', "{{%catalog_product}}",
            'currency_old', '{{%money_currency}}', 'code', 'RESTRICT', 'RESTRICT'
        );
        $this->addForeignKey(
            'catalog_product_currency_additional__money_currency', "{{%catalog_product}}",
            'currency_additional', '{{%money_currency}}', 'code', 'RESTRICT', 'RESTRICT'
        );

    }

    public function down()
    {
        $this->dropForeignKey("catalog_product_created_by", "{{%catalog_product}}");
        $this->dropForeignKey("catalog_product_updated_by", "{{%catalog_product}}");
        $this->dropForeignKey("catalog_product_tree_id_cms_tree", "{{%catalog_product}}");

        $this->dropForeignKey("catalog_product_currency__money_currency", "{{%catalog_product}}");
        $this->dropForeignKey("catalog_product_currency_procurement__money_currency", "{{%catalog_product}}");
        $this->dropForeignKey("catalog_product_currency_old__money_currency", "{{%catalog_product}}");
        $this->dropForeignKey("catalog_product_currency_additional__money_currency", "{{%catalog_product}}");

        $this->dropTable("{{%catalog_product}}");
    }
}
