<?php
/**
 * @author Semenov Alexander <semenov@skeeks.com>
 * @link http://skeeks.com/
 * @copyright 2010 SkeekS (СкикС)
 * @date 09.03.2015
 */
namespace skeeks\modules\cms\catalog\models;

use skeeks\cms\base\db\ActiveRecord;
use skeeks\cms\models\behaviors\CanBeLinkedToModel;
use skeeks\cms\models\behaviors\CanBeLinkedToTree;
use skeeks\cms\models\behaviors\HasDescriptionsBehavior;
use skeeks\cms\models\behaviors\HasPageOptions;
use skeeks\cms\models\behaviors\HasProperties;
use skeeks\cms\models\behaviors\HasStatus;
use skeeks\cms\models\behaviors\Implode;
use skeeks\cms\models\behaviors\TimestampPublishedBehavior;
use skeeks\cms\models\Core;
use skeeks\cms\models\PageAdvanced;
use skeeks\cms\models\PropertiesValidateModel;
use skeeks\cms\models\Tree;
use skeeks\modules\cms\money\Currency;
use skeeks\modules\cms\money\Money;
use Yii;
use yii\web\ErrorHandler;

/**
 * This is the model class for table "catalog_product".
 *
 * @property integer $id
 * @property integer $created_by
 * @property integer $updated_by
 * @property integer $created_at
 * @property integer $updated_at
 * @property integer $published_at
 * @property string $name
 * @property string $description_short
 * @property string $description_full
 * @property string $files
 * @property string $seo_page_name
 * @property integer $count_comment
 * @property integer $count_subscribe
 * @property string $users_subscribers
 * @property integer $count_vote
 * @property integer $result_vote
 * @property string $users_votes_up
 * @property string $users_votes_down
 * @property integer $status
 * @property integer $status_adult
 * @property string $page_options
 * @property integer $tree_id
 * @property string $tree_ids
 * @property string $type
 * @property double $availability
 * @property string $ammount
 * @property string $currency
 * @property string $ammount_procurement
 * @property string $currency_procurement
 * @property string $ammount_old
 * @property string $currency_old
 * @property string $ammount_additional
 * @property string $currency_additional
 * @property string $article
 * @property double $weight
 * @property double $availability_reserved
 */
class Product extends PageAdvanced
{
    public $viewPageTemplate = "catalog/product/view";

    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '{{%catalog_product}}';
    }


    /**
     * @return array
     */
    public function behaviors()
    {
        return array_merge(parent::behaviors(), [
            CanBeLinkedToTree::className(),
            TimestampPublishedBehavior::className() => TimestampPublishedBehavior::className(),
            HasPageOptions::className() => HasPageOptions::className(),
            HasProperties::className() => HasProperties::className(),
        ]);
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return array_merge(parent::attributeLabels(), [
            'type'          => Yii::t('app', 'Publication type'),
            'tree_ids'      => Yii::t('app', 'Разделы'),
            'tree_id'       => Yii::t('app', 'Главный раздел'),
            'page_options'  => Yii::t('app', 'Дополнительные свойства'),
            'published_at'  => Yii::t('app', 'Дата публикации'),
            'availability'  => "В наличии",
            'availability_reserved'  => "В резерве",
            'weight'  => "Вес",
            'article'       => "Артикул",
        ]);
    }

    public function scenarios()
    {
        $scenarios = parent::scenarios();

        $scenarios[self::SCENARIO_DEFAULT];

        $scenarios['create'] = array_merge($scenarios[self::SCENARIO_DEFAULT], ['!multiPageOptions']);
        $scenarios['update'] = array_merge($scenarios[self::SCENARIO_DEFAULT], ['!multiPageOptions']);

        return $scenarios;
    }

    public $multiPageOptions = [];
    /**
     * @inheritdoc
     */
    public function rules()
    {
        return array_merge(parent::rules(), [
            [['name'], 'required'],
            [['article'], 'unique'],
            [['availability', 'weight', 'availability_reserved'], 'number'],
            [['type', 'ammount', 'currency', 'ammount_procurement', 'currency_procurement', 'ammount_old', 'currency_old', 'ammount_additional', 'currency_additional', 'article'], 'string'],
            [['published_at', 'tree_id'], 'integer'],
            [['tree_ids', 'page_options', 'multiPageOptions'], 'safe'],

            ['weight', 'default', 'value' => function($model, $attribute)
            {
                return 0;
            }],

            ['availability_reserved', 'default', 'value' => function($model, $attribute)
            {
                return 0;
            }],
        ]);
    }

    /**
     * @return null|ModelType
     */
    public function getType()
    {
        if ($this->type)
        {
            return \Yii::$app->registeredModels->getDescriptor($this)->getTypes()->getComponent($this->type);
        }

        return null;
    }


    /**
     * @return null|Tree
     */
    public function fetchMainTree()
    {
        if ($treeId = $this->getMainTreeId())
        {
            return Tree::find()->where(['id' => $treeId])->limit(1)->one();
        }

        return null;
    }
    /**
     * @return int
     */
    public function getMainTreeId()
    {
        return (int) array_shift($this->getTreeIds());
    }

    /**
     * @return array
     */
    public function getTreeIds()
    {
        $result = [];

        if ($this->tree_id)
        {
            $result[] = (int) $this->tree_id;
        }

        if ($this->tree_ids)
        {
            $result = array_merge($result, $this->tree_ids);
        }

        return array_unique($result);
    }

    /**
     * @return Tree[]
     */
    public function fetchTrees()
    {
        if ($treeIds = $this->getTreeIds())
        {
            return Tree::find()->where(['id' => $treeIds])->all();
        }

        return [];
    }

    /**
     * @return Money
     */
    public function cost()
    {
        if ($this->ammount)
        {
            return Money::fromString((string) $this->ammount, $this->currency());
        }
        return Money::fromString((string) $this->ammount, $this->currency());
    }

    /**
     * @return Currency
     */
    public function currency()
    {
        if ($currency = $this->getAttribute($this->currency))
        {
            return Currency::getInstance($currency);
        }

        return Currency::getInstance(\Yii::$app->money->currency);
    }

    /**
     * Свойства группы
     *
     * @return \yii\db\ActiveQuery
     */
    public function findPropertyValues()
    {
        return $this->hasMany(ProductPropertyMap::className(), ['product_id' => 'id']);
    }

    /**
     * Свойства группы
     *
     * @return ProductPropertyMap[]
     */
    public function fetchPropertyValues()
    {
        return $this->findPropertyValues()->all();
    }


    /**
     * Значения связанных свойств
     * @return  \yii\db\ActiveQuery
     */
    public function getPropertyValues()
    {
        return $this->hasMany(ProductPropertyMap::className(), ['product_id' => 'id']);
    }


    /**
     * Свойства связанные с продуктом
     * @return  \yii\db\ActiveQuery
     */
    public function getProperties()
    {
        return $this->hasMany(ProductProperty::className(), ['id' => 'property_id'])
                    ->via('propertyValues');
    }

    /**
     * @return array|\yii\db\ActiveRecord[]
     */
    public function getPropertiesAll()
    {
        return ProductProperty::find()->active()->orderBy("priority DESC")->all();
    }

    /**
     * @param $id
     * @return array|null|\yii\db\ActiveRecord
     */
    public function findPropertyValue($id)
    {
        return $this->getPropertyValues()->where(['property_id' => $id]);
    }

    /**
     * @return string
     */
    public function renderPropertiesForm()
    {
        try
        {
            return \Yii::$app->view->render('@skeeks/modules/cms/catalog/views/blank-form', [
                'modelWithProperties'   => $this,
                'validateModel'         => $this->createPropertiesValidateModel()
            ]);

        } catch (\Exception $e)
        {
            ob_end_clean();
            ErrorHandler::convertExceptionToError($e);
            return 'Ошибка рендеринга формы: ' . $e->getMessage();
        }
    }

    /**
     * @return PropertiesValidateModel
     */
    public function createPropertiesValidateModel()
    {
        return new PropertiesValidateModel([
            'modelWithProperties' => $this
        ]);
    }
}