<?php
/**
 * @author Semenov Alexander <semenov@skeeks.com>
 * @link http://skeeks.com/
 * @copyright 2010 SkeekS (СкикС)
 * @date 10.03.2015
 */
namespace skeeks\modules\cms\catalog\models;

use skeeks\cms\base\PropertyType;
use skeeks\cms\base\propertyTypes\PropertyTypeStandartElement;
use skeeks\cms\components\Cms;
use skeeks\modules\cms\catalog\query\ProductPropertyQuery;
use \Yii as Yii;
use skeeks\cms\base\db\ActiveRecord;
use skeeks\cms\models\behaviors\HasDescriptionsBehavior;
use skeeks\cms\models\behaviors\HasFiles;
use skeeks\cms\models\behaviors\HasStatus;
use skeeks\cms\models\behaviors\Implode;
use skeeks\cms\models\Core;
use yii\base\Model;
use yii\db\BaseActiveRecord;
use yii\helpers\ArrayHelper;
use yii\widgets\ActiveForm;

/**
 * This is the model class for table "{{%catalog_product_property}}".
 *
 * @property integer $id
 * @property integer $created_by
 * @property integer $updated_by
 * @property integer $created_at
 * @property integer $updated_at
 * @property string $name
 * @property string $code
 * @property integer $group_id
 * @property string $active
 * @property integer $priority
 * @property string $default_value
 * @property string $property_type
 * @property integer $row_count
 * @property integer $col_count
 * @property string $list_type
 * @property string $multiple
 * @property string $file_type
 * @property integer $multiple_cnt
 * @property string $tmp_id
 * @property string $with_description
 * @property string $searchable
 * @property string $filtrable
 * @property string $smart_filtrable
 * @property string $is_required
 * @property integer $version
 * @property string $component
 * @property string $component_settings
 * @property string $hint
 */
class ProductProperty extends Core
{

    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '{{%catalog_product_property}}';
    }

    public function init()
    {
        parent::init();

        $this->on(BaseActiveRecord::EVENT_BEFORE_INSERT,    [$this, "processBeforeSave"]);
        $this->on(BaseActiveRecord::EVENT_BEFORE_UPDATE,    [$this, "processBeforeSave"]);
    }

    public function processBeforeSave()
    {
        if ($this->component)
        {
            /**
             * @var $propertyType PropertyType
             */
            $propertyTypeClassName = $this->component;
            $propertyType = new $propertyTypeClassName();

            $this->property_type    = $propertyType->code;
            $this->multiple         = $propertyType->multiple ? Cms::BOOL_Y :  Cms::BOOL_N;
        }
    }

    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return array_merge(parent::behaviors(), [
        ]);
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return array_merge(parent::rules(), [
            [['created_by', 'updated_by', 'created_at', 'updated_at', 'group_id', 'priority', 'row_count', 'col_count', 'multiple_cnt', 'version'], 'integer'],
            [['name', 'component'], 'required'],
            [['default_value', 'component_settings'], 'string'],
            [['name', 'component', 'hint'], 'string', 'max' => 255],
            [['code'], 'string', 'max' => 32],
            [['active', 'property_type', 'list_type', 'multiple', 'with_description', 'searchable', 'smart_filtrable', 'filtrable', 'is_required'], 'string', 'max' => 1],
            [['file_type'], 'string', 'max' => 200],
            [['tmp_id'], 'string', 'max' => 40],
            [['code'], 'unique'],
            ['code', 'default', 'value' => function($model, $attribute)
            {
                return "sx_auto_" . md5(rand(1, 10) . time());
            }],
            ['priority', 'default', 'value' => function($model, $attribute)
            {
                return 500;
            }],
        ]);
    }

    public function scenarios()
    {
        $scenarios = parent::scenarios();

        $scenarios['create'] = $scenarios[self::SCENARIO_DEFAULT];
        $scenarios['update'] = $scenarios[self::SCENARIO_DEFAULT];

        return $scenarios;
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return  array_merge(parent::attributeLabels(), [
            'id' => Yii::t('app', 'ID'),
            'created_by' => Yii::t('app', 'Created By'),
            'updated_by' => Yii::t('app', 'Updated By'),
            'created_at' => Yii::t('app', 'Created At'),
            'updated_at' => Yii::t('app', 'Updated At'),
            'name' => Yii::t('app', 'Name'),
            'code' => Yii::t('app', 'Code'),
            'group_id' => Yii::t('app', 'Group ID'),
            'active' => Yii::t('app', 'Active'),
            'priority' => Yii::t('app', 'Priority'),
            'default_value' => Yii::t('app', 'Default Value'),
            'property_type' => Yii::t('app', 'Тип свойства'),
            'row_count' => Yii::t('app', 'Row Count'),
            'col_count' => Yii::t('app', 'Col Count'),
            'list_type' => Yii::t('app', 'List Type'),
            'multiple' => Yii::t('app', 'Multiple'),
            'file_type' => Yii::t('app', 'File Type'),
            'multiple_cnt' => Yii::t('app', 'Количество полей для ввода новых множественных значений'),
            'tmp_id' => Yii::t('app', 'Tmp ID'),
            'with_description' => Yii::t('app', 'Выводить поле для описания значения'),
            'searchable' => Yii::t('app', 'Значения свойства участвуют в поиске'),
            'filtrable' => Yii::t('app', 'Выводить на странице списка элементов поле для фильтрации по этому свойству'),
            'smart_filtrable' => Yii::t('app', 'Показывать в умном фильтре'),
            'is_required' => Yii::t('app', 'Is Required'),
            'version' => Yii::t('app', 'Version'),
            'component' => Yii::t('app', 'Тип'),
            'component_settings' => Yii::t('app', 'Настройки'),
            'hint' => Yii::t('app', 'Hint'),
        ]);
    }


    /**
     * @return \yii\db\ActiveQuery
     */
    public function findGroup()
    {
        return $this->hasOne(ProductPropertyGroup::className(), ['id' => 'group_id']);
    }

    /**
     * @return ProductPropertyGroup
     */
    public function fetchGroup()
    {
        return $this->findGroup()->one();
    }



    /**
     * @param ActiveForm $activeForm
     * @param Model $model
     * @return mixed
     */
    public function renderActiveForm(ActiveForm $activeForm, Model $model)
    {
        $elementClass   = $this->component;

        /**
         * @var $propertyType PropertyType
         */
        $propertyType = new $elementClass([
            'model'         => $model,
            'property'      => $this,
            'activeForm'    => $activeForm,
        ]);

        return $propertyType->renderForActiveForm();
    }


    /**
     * @return array
     */
    public function rulesForActiveForm()
    {
        $result = [];

        $rules = [];

        if ($this->is_required == Cms::BOOL_Y)
        {
            $rules = ['required'];
        }

        if ((array) $rules)
        {
            foreach ((array) $rules as $ruleCode)
            {
                $result[] = [[$this->getFormAttribute()], $ruleCode];
            }
        } else
        {
            $result[] = [[$this->getFormAttribute()], 'safe'];
        }

        return $result;
    }

    /**
     * @return string
     */
    public function getFormAttribute()
    {
        return "property_" . $this->id;
    }

    /**
     * @param $modelWhithProperties
     * @return mixed
     */
    public function value($modelWhithProperties)
    {
        if ($this->multiple == "Y")
        {
            if ($values = $modelWhithProperties->findPropertyValue($this->id)->all())
            {
                return ArrayHelper::map($values, "id", "value");
            } else
            {
                return [];
            }
        } else
        {
            if ($value = $modelWhithProperties->findPropertyValue($this->id)->one())
            {
                return $value->value;
            } else
            {
                return null;
            }
        }
    }

    /**
     * @param $modelWhithProperties
     * @param $value
     * @return $this
     */
    public function saveValue($modelWhithProperties, $value)
    {
        if ($this->multiple == "Y")
        {
            $propertyValues = $modelWhithProperties->getPropertyValues()->where(['property_id' => $this->id])->all();
            if ($propertyValues)
            {
                foreach ($propertyValues as $pv)
                {
                    $pv->delete();
                }
            }

            $values = (array) $value;

            if ($values)
            {
                foreach ($values as $key => $value)
                {
                    $productPropertyValue = new ProductPropertyMap([
                        'product_id'    => $modelWhithProperties->id,
                        'property_id'   => $this->id,
                        'value'         => $value,
                    ]);

                    $productPropertyValue->save(false);
                }
            }

        } else
        {
            /**
             * @var $propertyValue ProductPropertyMap
             */
            if ($productPropertyValue = $modelWhithProperties->getPropertyValues()->where(['property_id' => $this->id])->one())
            {
                $productPropertyValue->value = $value;
            } else
            {
                $productPropertyValue = new ProductPropertyMap([
                    'product_id' => $modelWhithProperties->id,
                    'property_id' => $this->id,
                    'value' => $value,
                ]);
            }

            $productPropertyValue->save();
        }

        return $this;
    }




    /**
     * @return \yii\db\ActiveQuery
     */
    public function getProductPropertyEnums()
    {
        return $this->hasMany(ProductPropertyEnum::className(), ['property_id' => 'id']);
    }
}