<?php
/**
 * @author Semenov Alexander <semenov@skeeks.com>
 * @link http://skeeks.com/
 * @copyright 2010 SkeekS (СкикС)
 * @date 10.03.2015
 */
namespace skeeks\modules\cms\catalog\models;

use skeeks\cms\base\db\ActiveRecord;
use skeeks\cms\models\behaviors\HasDescriptionsBehavior;
use skeeks\cms\models\behaviors\HasFiles;
use skeeks\cms\models\behaviors\HasStatus;
use skeeks\cms\models\behaviors\Implode;
use skeeks\cms\models\Core;

/**
 * Class ProductPropertyMap
 * @package skeeks\modules\cms\catalog\models
 */
class ProductPropertyMap extends Core
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '{{%catalog_product_property_map}}';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return array_merge(parent::rules(), [
            [['value', 'product_id', 'property_id'], 'required'],
            [['value'], 'string'],
            [['product_id', 'property_id'], 'unique', 'targetAttribute' => ['product_id', 'property_id'], 'message' => 'Это свойство уже привязано к товару'],
        ]);
    }

    public function scenarios()
    {
        $scenarios = parent::scenarios();

        $scenarios['create'] = $scenarios[self::SCENARIO_DEFAULT];
        $scenarios['update'] = $scenarios[self::SCENARIO_DEFAULT];

        return $scenarios;
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return  array_merge(parent::attributeLabels(), [
            'id' => \Yii::t('app', 'ID'),
            'name' => \Yii::t('app', 'Name'),
            'value' => \Yii::t('app', 'Значение'),
            'product_id' => \Yii::t('app', 'Товар'),
            'property_id' => \Yii::t('app', 'Свойство товара'),
        ]);
    }


    /**
     * @return \yii\db\ActiveQuery
     */
    public function findProductProperty()
    {
        return $this->hasOne(ProductProperty::className(), ['id' => 'property_id']);
    }

    /**
     * @return ProductProperty
     */
    public function fetchProductProperty()
    {
        return $this->findProductProperty()->one();
    }


    /**
     * @return \yii\db\ActiveQuery
     */
    public function findProduct()
    {
        return $this->hasOne(ProductProperty::className(), ['id' => 'product_id']);
    }

    /**
     * @return Product
     */
    public function fetchProduct()
    {
        return $this->findProduct()->one();
    }


    /**
     * @return \yii\db\ActiveQuery
     */
    public function getProduct()
    {
        return $this->hasOne(Product::className(), ['id' => 'product_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getProperty()
    {
        return $this->hasOne(ProductProperty::className(), ['id' => 'property_id']);
    }

}