<?php
/**
 * Publications
 *
 * @author Semenov Alexander <semenov@skeeks.com>
 * @link http://skeeks.com/
 * @copyright 2010-2014 SkeekS (Sx)
 * @date 08.12.2014
 * @since 1.0.0
 */
namespace skeeks\modules\cms\catalog\widgets\products;

use skeeks\cms\base\Widget;
use skeeks\cms\models\Publication;
use skeeks\cms\models\Search;
use skeeks\cms\models\Tree;
use skeeks\cms\widgets\base\hasModels\WidgetHasModels;
use skeeks\cms\widgets\base\hasModelsSmart\WidgetHasModelsSmart;
use skeeks\cms\widgets\WidgetHasTemplate;
use Yii;
use yii\data\Pagination;
use yii\helpers\ArrayHelper;

/**
 * Class Publications
 * @package skeeks\cms\widgets\PublicationsAll
 */
class Products extends WidgetHasModelsSmart
{
    public $modelClassName          = '\skeeks\modules\cms\catalog\models\Product';

    /**
     * @var null|string
     */
    public $enableSubTree           = 0;
    public $enableSubTreeAll        = 0;
    public $title                   = '';
    public $types                   = [];
    public $statuses                = [];
    public $statusesAdults          = [];
    public $useCurrentTree          = 0;
    public $tree_ids                = [];

    public function rules()
    {
        return ArrayHelper::merge(parent::rules(), [
            [['title'], 'string'],
            [['types', 'statuses', 'statusesAdults', 'useCurrentTree', 'tree_ids'], 'safe'],
            [['enableSubTree', 'enableSubTreeAll'], 'integer'],
        ]);
    }

    public function attributeLabels()
    {
        return ArrayHelper::merge(parent::attributeLabels(), [
            'title'                         => 'Название виджета',
            'types'                         => 'Типы публикаций',
            'tree_ids'                      => 'Разделы сайта',
            'statuses'                      => 'Статусы',
            'statusesAdults'                => 'Статусы приватности',
            'enableSubTree'                 => 'Искать в дочерних подразделах (1 уровень вложенности)',
            'enableSubTreeAll'              => 'Искать во всех подразделах (вся вложенность)',
            'useCurrentTree'                => 'Добавлять условия выбора записей, страницы где находится этот виджет',
        ]);
    }

    public function buildSearch()
    {
        parent::buildSearch();
        //print_r($this->attributes);

        $dataProvider = $this->getSearch()->getDataProvider();
        $find = $dataProvider->query;

        $ids = [];
        //Используем текущий раздел
        if ($this->useCurrentTree)
        {
            $tree = \Yii::$app->cms->getCurrentTree();
            if ($tree)
            {
                if ($this->enableSubTreeAll)
                {
                    $ids = $tree->fetchChildrensAllIds();

                } else if ($this->enableSubTree)
                {
                    $ids = $tree->fetchChildrensIds();
                }

                $ids[] = $tree->id;

            }

        } else
        {
            if ($this->tree_ids)
            {
                $mainIds    = (array) $this->tree_ids;
                $ids        = $mainIds;

                if ($this->enableSubTreeAll)
                {
                    $trees = Tree::find()->where(['id' => $mainIds])->all();
                    if ($trees)
                    {
                        foreach ($trees as $tree)
                        {
                            if ($tree->hasChildrens())
                            {
                                if ($childrens = $tree->fetchChildrensAll())
                                {
                                    foreach ($childrens as $chidren)
                                    {
                                        $ids[] = $chidren->id;
                                    }
                                }
                            }
                        }
                    }
                } else if ($this->enableSubTree)
                {
                    $trees = Tree::find()->where(['id' => $mainIds])->all();
                    if ($trees)
                    {
                        foreach ($trees as $tree)
                        {
                            if ($tree->hasChildrens())
                            {
                                if ($childrens = $tree->fetchChildrens())
                                {
                                    foreach ($childrens as $chidren)
                                    {
                                        $ids[] = $chidren->id;
                                    }
                                }
                            }
                        }
                    }
                }

                $ids = array_unique($ids);
            }
        }

        if ($ids)
        {
            foreach ($ids as $id)
            {
                $find->orWhere("(FIND_IN_SET ('{$id}', tree_ids) or tree_id = '{$id}')");
            }
        }



        if ($this->statuses)
        {
            $find->andWhere(['status' => $this->statuses]);
        }

        if ($this->statusesAdults)
        {
            $find->andWhere(['status_adult' => $this->statuses]);
        }

        if ($this->types)
        {
            $find->andWhere(['type' => $this->types]);
        }


        if ($this->createdBy)
        {
            $find->andWhere(['created_by' => $this->createdBy]);
        }


        if ($this->updatedBy)
        {
            $find->andWhere(['updated_by' => $this->updatedBy]);
        }

        return $this;
    }

    /**
     * Подготовка данных для шаблона
     * @return $this
     */
    public function bind()
    {
        /*$this->buildSearch();
        $this->getSearch()->search(\Yii::$app->request->queryParams);*/
        parent::bind();

    }




    /**
     * @return array|null|Tree
     */
    public function fetchFirstTree()
    {
        if ($id = $this->getFirstTreeId())
        {
            return Tree::find()->where(['id' => $id])->one();
        } else
        {
            return null;
        }
    }

    /**
     * @return int
     */
    public function getFirstTreeId()
    {
        if ($this->tree_ids)
        {
            return (int) array_shift($this->tree_ids);
        } else
        {
            return 0;
        }
    }
}
